@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Nominum, Inc.
+# Copyright (C) 2006, 2007, 2009-2011 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -37,8 +37,13 @@ def from_address(text):
     @rtype: dns.name.Name object
     """
     try:
-        parts = list(dns.ipv6.inet_aton(text).encode('hex_codec'))
-        origin = ipv6_reverse_domain
+        v6 = dns.ipv6.inet_aton(text)
+        if dns.ipv6.is_mapped(v6):
+            parts = ['%d' % ord(byte) for byte in v6[12:]]
+            origin = ipv4_reverse_domain
+        else:
+            parts = list(v6.encode('hex_codec'))
+            origin = ipv6_reverse_domain
     except:
         parts = ['%d' % ord(byte) for byte in dns.ipv4.inet_aton(text)]
         origin = ipv4_reverse_domain
@@ -72,4 +77,4 @@ def to_address(name):
         # run through inet_aton() to check syntax and make pretty.
         return dns.ipv6.inet_ntoa(dns.ipv6.inet_aton(text))
     else:
-        raise dns.exception.SyntaxError, 'unknown reverse-map address family'
+        raise dns.exception.SyntaxError('unknown reverse-map address family')
